import os

from langchain_openai import AzureChatOpenAI, ChatOpenAI

from parsera.utils import singleton


@singleton
class AzureModel(AzureChatOpenAI):
    def __init__(self, *args, **kwargs):
        super().__init__(
            azure_endpoint=os.getenv("AZURE_GPT_BASE_URL"),
            openai_api_version="2023-05-15",
            deployment_name=os.getenv("AZURE_GPT_DEPLOYMENT_NAME"),
            openai_api_key=os.getenv("AZURE_GPT_API_KEY"),
            openai_api_type="azure",
            temperature=0.0,
            timeout=120,
            *args,
            **kwargs,
        )


@singleton
class GPT4oMiniModel(ChatOpenAI):
    def __init__(self, *args, **kwargs):
        super().__init__(
            model="gpt-4o-mini",
            temperature=0.0,
            timeout=120,
            *args,
            **kwargs,
        )



import asyncio

from langchain_core.language_models import BaseChatModel

from parsera.engine.model import GPT4oMiniModel
from parsera.engine.simple_extractor import TabularExtractor
from parsera.page import fetch_page_content


class Parsera:
    def __init__(self, model: BaseChatModel | None = None):
        if model is None:
            self.model = GPT4oMiniModel()
        else:
            self.model = model

    async def _run(self, url: str, elements: dict) -> dict:
        content = await fetch_page_content(url=url)
        extractor = TabularExtractor(
            elements=elements, model=self.model, content=content
        )
        result = await extractor.run()
        return result

    def run(self, url: str, elements: dict) -> dict:
        return asyncio.run(self._run(url=url, elements=elements))



import json

from langchain_core.language_models import BaseChatModel
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.output_parsers import JsonOutputParser
from markdownify import markdownify

SIMPLE_EXTRACTOR_SYSTEM_PROMPT = """
Your goal is to find the elements from the webpage content and return them in json format.
For example if user asks:
Return the following elements from the page content:
```
{
    "name": "name of the listing",
    "price": "price of the listing"
}
```
Make sure to return json with the list of corresponding values.
Output json:
```json
{
    "names": ["name1", "name2", "name3"],
    "price": ["100", "150", "300"],
}
```
"""

SIMPLE_EXTRACTOR_PROMPT_TEMPLATE = """
Having following page content:
```
{markdown}
```

Return the following elements from the page content:
```
{elements}
```

Output json:
"""


class JsonExtractor:
    system_prompt = SIMPLE_EXTRACTOR_SYSTEM_PROMPT
    prompt_template = SIMPLE_EXTRACTOR_PROMPT_TEMPLATE

    def __init__(self, elements: dict, model: BaseChatModel, content: str):
        self.elements = elements
        self.model = model
        self.content = content

    async def run(self) -> dict:
        markdown = markdownify(self.content)

        elements = json.dumps(self.elements)
        human_msg = self.prompt_template.format(markdown=markdown, elements=elements)
        messages = [
            SystemMessage(self.system_prompt),
            HumanMessage(human_msg),
        ]
        output = await self.model.ainvoke(messages)
        parser = JsonOutputParser()
        output_dict = parser.parse(output.content)
        return output_dict


TABULAR_EXTRACTOR_SYSTEM_PROMPT = """
Your goal is to find the elements from the webpage content and return them in json format.
For example if user asks:
Return the following elements from the page content:
```
{
    "name": "name of the listing",
    "price": "price of the listing"
}
```
Make sure to return json with the list of corresponding values.
Output json:
```json
[
    {"name": "name1", "price": "100"},
    {"name": "name2", "price": "150"},
    {"name": "name3", "price": "300"},
]
```

If users asks for a single field:
Return the following elements from the page content:
```
{
    "link": "link to the listing",
}
```
Make sure to return json with only this field
Output json:
```json
[
    {"link": "https://example.com/link1"},
    {"link": "https://example.com/link2"},
    {"link": "https://example.com/link3"},
]
```

"""


class TabularExtractor(JsonExtractor):
    system_prompt = TABULAR_EXTRACTOR_SYSTEM_PROMPT



from playwright.async_api import async_playwright
from playwright_stealth import stealth_async


async def fetch_page_content(url: str) -> str:
    async with async_playwright() as p:
        # Launch the browser
        browser = await p.chromium.launch(headless=True)
        # Open a new browser context
        context = await browser.new_context()
        # Open a new page
        page = await context.new_page()
        await stealth_async(page)

        # Navigate to the URL
        # await page.route("**/*.{png,jpg,jpeg}", lambda route: route.abort()) # Can speed up requests
        await page.goto(url)

        # Wait for the content to be dynamically loaded
        await page.wait_for_load_state("domcontentloaded")
        # Get the page content
        content = await page.content()

        # Close the browser
        await browser.close()

        return content
